@@ -1,6 +1,6 @@
-skip()
+# skip()
 skip_on_cran()
-skip_on_ci()
+# skip_on_ci()
 
 
 a <- tophits("ieu-a-2")
